/*
 * Module to allow the parser to read from a file
 */
 
#include <stdio.h>

#ifndef _NFILE
# define _NFILE 20
#endif

/*
 * We define a stack of files which we have partially read.  When we
 * want to read a new file, read_file() is called with its name.  The
 * current file's info is pushed on the stack.  yywrap() pops the stack.
 * If the stack is empty already, yywrap() returns 1, indicating to lex
 * that there is no more input, causing the parser to exit, causing the
 * program to exit.
 */
#define	NOMOREFILES	(-2)
static int file_stack = NOMOREFILES;
extern FILE *yyin;
extern int yylineno;
static char *current_input;	/* Name of current input file or NULL */
static struct {
    FILE *file;			/* an input stream */
    int lineno;			/* line number on that stream */
    char *name;			/* Name of the file */
} input_files[_NFILE];

extern char yysbuf[], *yysptr;


/*
 * Routine called on end of file
 */
int yywrap()
{
    (void) fclose(yyin);
    if (file_stack <= NOMOREFILES + 1) return(1);	/* End of execution */
    yyin = input_files[file_stack].file;
    current_input = input_files[file_stack].name;
    yylineno = input_files[file_stack--].lineno;
    yysptr = yysbuf;
    
    return(0);
}


/*
 * Routine to "source" "file"
 */
void read_file(file)
char *file;		/* File to open */
{
    if (++file_stack >= 0) {
	input_files[file_stack].file = yyin;
	input_files[file_stack].lineno = yylineno;
	input_files[file_stack].name = current_input;
    }
    if (! (yyin = fopen(file, "r"))) {
	perror(file);
	if (file_stack >= 0) yyin = input_files[file_stack].file;
	--file_stack;
	return;
    }
    else {			/* Starting a new file */
	yylineno = 1;
	yysptr = yysbuf;
    }
    current_input = file;
}


/*
 * Error handling routine for parser.
 */
yyerror(s)
char *s;			/* Error message to print */
{
    if (current_input)
	fprintf(stderr, "Error, %s, line %d: %s\n",
		current_input, yylineno, s);
    else fprintf(stderr, "Error, line %d: %s\n", yylineno, s);
}


/*
 * Read input from stdin stream
 */
void read_stdin()
{
    if (++file_stack >= 0) {
	input_files[file_stack].file = yyin;
	input_files[file_stack].lineno = yylineno;
	input_files[file_stack].name = current_input;
    }
    yyin = stdin;
    yylineno = 1;
    yysptr = yysbuf;
    current_input = NULL;
}
