/*
 * Graph routine for parser for grapher program. 
 *
 */

#include <math.h>
#include <stdio.h>
#include <setjmp.h>
#include <strings.h>
#include "graph.h"
extern jmp_buf  jmpenv;
extern struct points_struct points[MAXSETS];
extern char     xlabel[256], ylabel[256];
extern int      tpic, print_symbols, label_last_point;
extern int      xaxis_type, yaxis_type, axis_type;
extern char     axis_label[256];
extern double  *xpoints_ptr, *ypoints_ptr;
extern double   aspect_ratio;
extern int      number_of_sets;
extern double   maxval, minval, xmax, xmin, ymax, ymin;
extern double   delx, dely, delval;
static double   circle_rad;

graph()
{
    double          inches_per_division;
    int             set, i, j;
    int             deltax, deltay;


    /* log correction: the max and min on a log scale must be logged,
     * unless they are derived from the data points (since the data points
     * have already been logged.) Here, if the max and/or min were specified,
     * they are logged.  An exception must be made for 0, since log10(0) is
     * undefined.  If the max and min are not specified, then their values
     * will be -999999 and 999999 respectively.
     * BS
     */
    if (xaxis_type == LOG){
	if (xmax != -999999 && xmax != 0) xmax = log10(xmax);
	if (xmin != 999999 && xmin != 0) xmin = log10(xmin);
    }
    if (yaxis_type == LOG){
	if (ymax != -999999 && ymax != 0) ymax = log10(ymax);
	if (ymin != 999999 && ymin != 0) ymin = log10(ymin);
    }

    
    /*        
     * find max and min if max and min are given, override if there are
     * points outside the graph  
     */
    
     for (set = 0; set <= number_of_sets; set++) {
	xpoints_ptr = &points[set].xpoints[0];
	ypoints_ptr = &points[set].ypoints[0];
	for (i = 1; i <= points[set].number_of_points; i++) {
	    if (xmax < *xpoints_ptr)
		xmax = *xpoints_ptr;
	    if (xmin > *xpoints_ptr)
		xmin = *xpoints_ptr;
	    xpoints_ptr++;
	    if (ymax < *ypoints_ptr)
		ymax = *ypoints_ptr;
	    if (ymin > *ypoints_ptr)
		ymin = *ypoints_ptr;
	    ypoints_ptr++;
	}
    }


    /* figure out scaling, but only if not specified */

    if (delx == -1) {
	delx = (xmax - xmin)/5.;
	if (delx > 1)
		{
		delx = ceil(delx);
		delx = ceil(delx / pow((double) 10,
	    	floor(log10(delx))))
	    	* pow((double) 10, floor(log10(delx)));
		}
	/* if delx is > 5, make them multiple of 5 */
	if (delx > 5)
	    delx = floor(delx / (double) 5) * (double) 5;
    }
    if (dely == -1) {
	dely = (ymax - ymin) / 5.;
	if (dely > 1)
		{
		dely = ceil(dely);
		dely = ceil(dely / pow((double) 10,
			floor(log10(dely))))
	    	* pow((double) 10, floor(log10(dely)));
		}
	/* if dely is > 5, make them multiple of 5 */
	if (dely > 5)
	    dely = floor(dely / (double) 5) * (double) 5;
    }

    
    /* correction for LOG type: deln needs to be removed, for it makes no
       sense on a logrithmic scale.  Because deln is always used as a
       divisor, setting it to 1 effectively removes it.  BS */
    if (xaxis_type == LOG) delx = 1;
    if (yaxis_type == LOG) dely = 1;

    
    /* figure out lowest and highest points displayed */

    xmin = floor(xmin / delx) * delx;
    xmax = ceil(xmax / delx) * delx;
    ymin = floor(ymin / dely) * dely;
    ymax = ceil(ymax / dely) * dely;
    deltax = (int) fabs((xmax - xmin) / delx);
    deltay = (int) fabs((ymax - ymin) / dely);

    
    /* DEBUG code -- dump interesting information.  BS 
    fprintf(stderr,"xmax = %g, xmin = %g\n",xmax,xmin);
    fprintf(stderr,"delx = %g, deltax = %d\n\n",delx,deltax);
    fprintf(stderr,"ymax = %g, ymin = %g\n",ymax,ymin);
    fprintf(stderr,"dely = %g, deltay = %d\n",dely,deltay); 
    */


    
    /* maintain required aspect ration */

    inches_per_division = (aspect_ratio * (double) deltax) / (double) deltay;
    circle_rad = ((double) deltax) / 100.;

    /* print header for eqn */

    if (!tpic) {
	puts(".EQ");
	puts("delim $$");
	puts(".EN");
    }

    /* default picture size is 6i */
    puts(".PS 6i");
    printf("G: box ht %gi wid %di invis\n",
	deltay * inches_per_division, deltax);
    printf("\"%s\" at G.nw +(0, %gi) center\n", ylabel, 6 * circle_rad);
    printf("\"%s\" at G.se -(0, %gi) rjust\n", xlabel, 6 * circle_rad);

    /* draw the x axis */

    for (j = 0; j <= deltax; j++) {
	printf("move to G.sw +(%di, %gi)\n", j, deltay * inches_per_division);
	printf("line down %gi\n", (deltay * inches_per_division) + circle_rad);
	printf("move down %gi\n", 2 * circle_rad);
	if (xaxis_type == LOG) {
	    if (tpic)
		printf("\"$10^{%d}$\" center\n", j);
	    else
		printf("\"$10 sup %d$\" center\n", j);
	}
	else
	    printf("\"%g\" center\n", xmin + (double) j * delx);
	if (xaxis_type == LOG)  /* add hash marks */
	    if (j < deltax)
		for (i = 2; i <= 9; i++) {
		    /* only add a mark if there is space in between it and
		       the next one.  (same change for y axis) BS */
		    if (log10((double)i+1)-log10((double)i) > .05) {
			printf("move to G.sw +(%gi, %gi)\n",
			log10((double) i) + j, circle_rad);
		        printf("line down %gi\n", 2 * circle_rad);
		    }
		}
    }


    
    /* draw the y axis */

    for (j = 0; j <= deltay; j++) {
	printf("move to G.sw +(%di, %gi)\n", deltax, j * inches_per_division);
	printf("line left %gi\n", deltax + circle_rad);
	printf("move left %gi\n", 2 * circle_rad);
	if (yaxis_type == LOG) {
	    if (tpic)
		printf("\"$10^{%d}$\" rjust\n", j);
	    else
		printf("\"$10 sup %d$\" rjust\n", j);
	}
	else
	    printf("\"%g\" rjust\n", ymin + (double) j * dely);
	if (yaxis_type == LOG)  /* add hash marks */
	    if (j < deltay)
		for (i = 2; i <= 9; i++){
		    if (log10((double)i+1)-log10((double)i) > .05) {
	                printf("move to G.sw +(%gi, %gi)\n",
			    circle_rad,
			    (log10((double) i) + j) * inches_per_division);
	    	            printf("line left %gi\n", 2 * circle_rad);
		    }
		}
    }

    /* plot the points */

    for (set = 0; set <= number_of_sets; set++) {
	if (print_symbols) {
	    gen_symbol(set);
	    printf("with .c at G.sw -(0i, %gi)\n", 7*circle_rad + (set*3*circle_rad));
	    printf("move to last [].c\n");
	    printf("move right 0.2i; \"%s\" ljust\n", points[set].pointlabel);
	}
	xpoints_ptr = &points[set].xpoints[0];
	ypoints_ptr = &points[set].ypoints[0];
	if (print_symbols) {
	    gen_symbol(set);
	    printf("with .c at G.sw +(%gi, %gi)\n",
		   (*xpoints_ptr++ - xmin) / delx,
		   (*ypoints_ptr++ - ymin) * inches_per_division / dely);
	}
	else
	    printf("move to G.sw +(%gi, %gi)\n",
		   (*xpoints_ptr++ - xmin) / delx,
		   (*ypoints_ptr++ - ymin) * inches_per_division / dely);
	for (i = 2; i <= points[set].number_of_points; i++) {
	    if (print_symbols)
		printf("line from last [].c");
	    else
		printf("line");

	    
	    /* DEBUG code -- display the data points' screen co-ord's
	    fprintf(stderr,"(%g,%g)\n",
		    (*xpoints_ptr - xmin)/delx,
		    (*ypoints_ptr - ymin) *inches_per_division / dely); 
	    */

	    
	    printf(" to G.sw +(%gi, %gi)\n",
		   (*xpoints_ptr - xmin) / delx,
		   (*ypoints_ptr - ymin) * inches_per_division / dely);
	    if (print_symbols) {
		gen_symbol(set);
		printf("with .c at G.sw +(%gi, %gi)\n",
		       (*xpoints_ptr++ - xmin) / delx,
		       (*ypoints_ptr++ - ymin) * inches_per_division / dely);
	    }
	    else
		printf("move to G.sw +(%gi, %gi)\n",
			(*xpoints_ptr++ - xmin) / delx,
			(*ypoints_ptr++ - ymin) * inches_per_division / dely);
	    if (i == points[set].number_of_points && label_last_point)
		printf("move right 0.1i; \"%s\" ljust\n",
			points[set].pointlabel);
	}
    }
    printf(".PE\n");
}


gen_symbol(x)
int x;
{
    switch (x) {
    case 0:
	printf("[circle rad %gi] ", circle_rad);
	break;
    case 1:
	printf("[box ht %gi wid %gi] ",
	    1.5 * circle_rad, 1.5 * circle_rad);
	break;
    case 2:
	printf("[line up %gi right %gi then down %gi right %gi then left %gi] ",
	    2 * circle_rad, circle_rad, 2 * circle_rad, circle_rad,
		2 * circle_rad);
	break;
    case 3:
	printf("[line up %gi right %gi then down %gi right %gi then down %gi left %gi then up %gi left %gi] ",
	    circle_rad, circle_rad, circle_rad, circle_rad,
		circle_rad, circle_rad, circle_rad, circle_rad);
	break;
    case 4:
	printf("[line down %gi right %gi then up %gi right %gi then left %gi] ",
	    2 * circle_rad, circle_rad, 2 * circle_rad,
	    circle_rad, 2 * circle_rad);
	break;
    case 5:
	printf("[line up %gi then down %gi right %gi then down %gi left %gi then up %gi] ",
	    circle_rad, circle_rad, circle_rad, circle_rad,
	    circle_rad, circle_rad);
	break;
    case 6:
	printf("[line up %gi right %gi then down %gi then left %gi up %gi] ",
	    circle_rad, circle_rad, 2*circle_rad, circle_rad, circle_rad);
	break;
    case 7:
	printf("[line right %gi then left %gi then up %gi then down %gi then left %gi then up %gi then right %gi then down %gi then left %gi] ",
	    2*circle_rad, circle_rad, circle_rad, 2*circle_rad, circle_rad,
	    2*circle_rad, 2*circle_rad, 2*circle_rad, circle_rad);
	break;
    case 8:
	printf("[line down %gi right %gi then up %gi left %gi then up %gi right %gi then down %gi left %gi] ",
	    2*circle_rad, 2*circle_rad, circle_rad, circle_rad, circle_rad,
	    circle_rad, 2*circle_rad, 2*circle_rad);
	break;
    }
}
