/*
 * Grammar for parser for grapher program.
 *
 */

%{
#include <math.h>
#include <stdio.h>
#include <setjmp.h>
#include <strings.h>
#include "graph.h"
extern jmp_buf jmpenv;
struct points_struct points[MAXSETS];
char xlabel[256], ylabel[256];
int xaxis_type, yaxis_type, axis_type;
char axis_label[256];
double *xpoints_ptr, *ypoints_ptr;
double aspect_ratio;
int number_of_sets = -1;
double maxval, minval, xmax, xmin, ymax, ymin;
double delx, dely, delval;
%}

%union {		/* Yacc value-stack elements */
    char *id;
    int lextoken;		/* No particular value returned by yylex() */
    float floating;		/* Floating-point value of a constant */
}

%token  <id>		String
%token	<floating>	Floating
%token	<lextoken>	data del label xaxis yaxis type min max log aspectratio 
%token	<lextoken>	newline eof

%start	input

%%

input:		graph_definition data_lists newline_option
	{
	graph();
	return(0);
	}
	;

newline_option:	newline
	|
	;

graph_definition: aspect_ratio_specification_option xaxis_specification_option
			yaxis_specification_option
	;

aspect_ratio_specification_option: aspect_ratio_specification
	|
		{
		aspect_ratio = DEFAULT_ASPECT_RATIO;
		}
	;

aspect_ratio_specification: aspectratio '=' Floating newline
		{
		aspect_ratio = $3;
		}
	;

xaxis_specification_option: xaxis_specification
	|	{
		xaxis_type = LINEAR;
		xmin = 999999;
		xmax = -999999;
		delx = -1;
		*xlabel = '\0';
		}
	;

yaxis_specification_option: yaxis_specification
	|	{
		yaxis_type = LINEAR;
		ymin = 999999;
		ymax = -999999;
		dely = -1;
		*ylabel = '\0';
		}
	;

xaxis_specification: xaxis ':' newline
		{
		axis_type = LINEAR;
		minval = 999999;
		maxval = -999999;
		delval = -1;
		*axis_label = '\0';
		}
		attribute_definition_list
		{
		xaxis_type = axis_type;
		xmin = minval;
		xmax = maxval;
		delx = delval;
		(void) strcpy (&xlabel[0], &axis_label[0]);
		}
	;

yaxis_specification: yaxis ':' newline
		{
		axis_type = LINEAR;
		minval = 999999;
		maxval = -999999;
		delval = -1;
		*axis_label = '\0';
		}
		attribute_definition_list
		{
		yaxis_type = axis_type;
		ymin = minval;
		ymax = maxval;
		dely = delval;
		(void) strcpy (&ylabel[0], &axis_label[0]);
		}
	;
 
attribute_definition_list: attribute_definition_list attribute_definition
	| attribute_definition
	;

attribute_definition: type '=' log  newline
		{
		axis_type = LOG;
		}
	| label '=' String newline
		{
		(void) strcpy(&axis_label[0], $3);
		}
	| min '=' Floating newline
		{
		minval = $3;
		}
	| max '=' Floating newline
		{
		maxval = $3;
		}
	| del '=' Floating newline
		{
		delval = $3;
		}
	;

data_lists: data_lists data_list
	| data_list
	;

data_list: data ':' newline 
		{
		if (number_of_sets++ >= MAXSETS)
			{
			printf ("Cannot handle %d sets of data, quitting\n", number_of_sets);
			return(0);
			}
		points[number_of_sets].number_of_points = 0;
		xpoints_ptr = &points[number_of_sets].xpoints[0];
		ypoints_ptr = &points[number_of_sets].ypoints[0];
		}
	label_option
	data_point_list
	label_option
	;

label_option: label '=' String newline
		{
		(void) strcpy(&(points[number_of_sets].pointlabel[0]),$3);
		}
	|
		{
		points[number_of_sets].pointlabel[0] = '\0';
		}
	;

data_point_list : data_point_list data_point
	| data_point
	;

data_point: Floating Floating newline
		{
		if (xaxis_type == LOG)
			*xpoints_ptr++ = log10((double) $1);
		else
			*xpoints_ptr++ = (double) $1;
		if (yaxis_type == LOG)
			*ypoints_ptr++ = log10((double) $2);
		else
			*ypoints_ptr++ = (double) $2;
		if (points[number_of_sets].number_of_points++ > MAXPOINTS)
			{
			printf("Cannot handle more than %d points, quitting\n", MAXPOINTS);
			return(0);
			}
		}
